
#include "LuaScript.h"

#include <Scripting/LuaVM.h>
#include <map>

#include "LuaStackWatcher.h"

template<typename _T>
inline void Lua::PassArgument(_T arg)
{	
	ASSERT(luaVM);

	++argCount;
	luaVM->Push(arg);
}

template<typename _T>
inline void Lua::PassArgument(_T arg, const Mortar::LuaObjectHandle& data)
{	
	ASSERT(luaVM);

	++argCount;
	luaVM->PushUserdataPtr(arg, data);
}

template<>
inline void Lua::PassArgument(void* userData)
{
	ASSERT(luaVM);

	++argCount;
	luaVM->PushLightUserdata(userData);
}

template<>
inline void Lua::PassArgument(Mortar::LuaObjectHandle data)
{
	ASSERT(luaVM);

	++argCount;
	data->Push();
}

template<typename _T>
inline _T Lua::GetReturnValue()
{
	ASSERT(luaVM);

	_T val;
	luaVM->Get(-1, val);
	retCountLua -= 1;
	luaVM->Pop(1);

	return val;
}

template<>
inline Vector2 Lua::GetReturnValue()
{
	ASSERT(luaVM);

	float y = GetReturnValue<float>();
	float x = GetReturnValue<float>();

	return Vector2(x, y);
}

template<typename _T>
inline void Lua::ReturnValue(_T val)
{
	ASSERT(luaVM != NULL);

	++retCount;
	luaVM->Push(val);
}

template<>
inline void Lua::ReturnValue(void* userData)
{
	ASSERT(luaVM != NULL);

	++retCount;
	luaVM->PushLightUserdata(userData);
}



/// <summary>
/// CallGlobalFunction (no parameters)
///  Wrapper for
///   Lua::SetGlobalFunction(functionName);
///   Lua::CallGlobalFunction(returnCount)
/// </summary>
inline bool Lua::CallGlobalFunction(const u32 returnCount, const char* className, const char* functionName)
{
	const std::string globalFunctionName = Mortar::StringFormatOut<std::string>("{0}.{1}", className, functionName);
	Lua::SetGlobalFunction(globalFunctionName.c_str());
	return Lua::CallGlobalFunction(returnCount);
}

/// <summary>
/// CallGlobalFunction (1 parameter)
///  Wrapper for
///   Lua::SetGlobalFunction(functionName);
///   Lua::PassArgument(param1)
///   Lua::CallGlobalFunction(returnCount)
/// </summary>
template<typename T1>
inline bool Lua::CallGlobalFunction(const u32 returnCount, const char* className, const char* functionName, T1 param1)
{
	const std::string globalFunctionName = Mortar::StringFormatOut<std::string>("{0}.{1}", className, functionName);
	Lua::SetGlobalFunction(globalFunctionName.c_str());
	Lua::PassArgument(param1);
	return Lua::CallGlobalFunction(returnCount);
}

/// <summary>
/// CallGlobalFunction (1 parameter)
///  Wrapper for
///   Lua::SetGlobalFunction(functionName);
///   Lua::PassArgument(param1)
///   Lua::PassArgument(param2)
///   Lua::CallGlobalFunction(returnCount)
/// </summary>
template<typename T1, typename T2>
inline bool Lua::CallGlobalFunction(const u32 returnCount, const char* className, const char* functionName, T1 param1, T2 param2)
{
	const std::string globalFunctionName = Mortar::StringFormatOut<std::string>("{0}.{1}", className, functionName);
	Lua::SetGlobalFunction(globalFunctionName.c_str());
	Lua::PassArgument(param1);
	Lua::PassArgument(param2);
	return Lua::CallGlobalFunction(returnCount);
}



/// <summary>
/// CallMemberFunction (no parameters)
///  Wrapper for
///   Lua::SetFunction(functionName);
///   Lua::PassArgument(instance, self);
///   Lua::CallFunction(returnCount)
/// </summary>
template<typename InstanceType>
inline bool Lua::CallMemberFunction(const u32 returnCount, InstanceType instance, const Mortar::LuaObjectHandle& self, const char* functionName)
{
	Lua::SetFunction(functionName);
	Lua::PassArgument(instance, self);
	return Lua::CallFunction(returnCount);
}

/// <summary>
/// CallMemberFunction (1 parameter)
///  Wrapper for
///   Lua::SetFunction(functionName);
///   Lua::PassArgument(instance, self);
///   Lua::PassArgument(param1)
///   Lua::CallFunction(returnCount)
/// </summary>
template<typename InstanceType, typename T1>
inline bool Lua::CallMemberFunction(const u32 returnCount, InstanceType instance, const Mortar::LuaObjectHandle& self, const char* functionName, T1 param1)
{
	Lua::SetFunction(functionName);
	Lua::PassArgument(instance, self);
	Lua::PassArgument(param1);
	return Lua::CallFunction(returnCount);
}

/// <summary>
/// CallMemberFunction (1 parameter)
///  Wrapper for
///   Lua::SetFunction(functionName);
///   Lua::PassArgument(instance, self);
///   Lua::PassArgument(param1)
///   Lua::PassArgument(param2)
///   Lua::CallFunction(returnCount)
/// </summary>
template<typename InstanceType, typename T1, typename T2>
inline bool Lua::CallMemberFunction(const u32 returnCount, InstanceType instance, const Mortar::LuaObjectHandle& self, const char* functionName, T1 param1, T2 param2)
{
	Lua::SetFunction(functionName);
	Lua::PassArgument(instance, self);
	Lua::PassArgument(param1);
	Lua::PassArgument(param2);
	return Lua::CallFunction(returnCount);
}


